﻿using System;
using System.Runtime.InteropServices; // For the DllImport of AIOWDM Driver API

namespace AIONet
{
    using TWinErrCode     = UInt32;
    using TBoardIndex     = UInt64;
    using TChannelIndex   = UInt32;

    public class AIO
    {
        #region Consts / Sentinel Values, Enums
        public const TWinErrCode ERROR_SUCCESS = 0;
        public const TBoardIndex biAuto = 0;
        public const TBoardIndex biNone = 0xFFFFFFFF_FFFFFFFF;
        public const TChannelIndex ciAll = 0xFFFF_FFFF;

        public enum Range : UInt32
        {
            NoChange   = 0xFFFFFFFF,
            Unipolar10 = 0x30313055, // FourCC("U010");
            Unipolar5  = 0x35303055, // FourCC("U005");
            Bipolar10  = 0x3031E142, // FourCC("B±10");
            Bipolar5   = 0x3530E142, // FourCC("B±05");
            Bipolar2_5 = 0xBD32E142, // FourCC("B±2½");
            Current420 = 0x30323449, // FourCC("I420"); // 4-20mA
            Code0 = 0,
            Code1 = 1,
            Code2 = 2,
            Code3 = 3,
            Code4 = 4,
            Code5 = 5,
            Code6 = 6,
            Code7 = 7,
        }

        public enum Sequencer : UInt32 { First = 0, Second = 1 }
        public enum IRQIndex : UInt32 { Only = 0 }
        public enum ChunkingType : UInt32 { Auto = 0, Size = 1, Rate = 1}
        public enum CallbackType : UInt32 { Raw = 0, Volts = 1, Current = 2}
        #endregion
        
        #region Board-level functions
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern UInt64 BRD_GetHighestBoardIndex();
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode BRD_GetNameW(TBoardIndex iBoard, UInt32 bufLen, IntPtr pNameBuf, ref UInt32 pNameLen);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode BRD_GetNameA(TBoardIndex iBoard, UInt32 bufLen, IntPtr pNameBuf, ref UInt32 pNameLen);

        /// <summary>
        /// TBD: Retrieve the friendly-name ("Model Number") for a given Board Index
        /// </summary>
        /// <param name="iBoard">the Board Index to query</param>
        /// <param name="Name">the destination string where the friendly name will be stored</param>
        /// <returns>TWinErrCode</returns>
        internal static TWinErrCode BRD_GetName(TBoardIndex iBoard, out string Name)
        {
            UInt32 Status;
            UInt32 NameSize = 64;
            IntPtr pNameStr = Marshal.AllocHGlobal((Int32)(NameSize + 1));

            Status = BRD_GetNameW(iBoard, NameSize,  pNameStr, ref NameSize);
            if (Status == 0)
            {
                Name = Marshal.PtrToStringUni(pNameStr);
            }
            else
            {
                Name = "error during BRD_GetName";
            }
            return Status;
        }

        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode BRD_GetSerialNumber(TBoardIndex iBoard, out UInt64 serialNumber);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode BRD_Reset(TBoardIndex iBoard);
        #endregion

        #region DAC functions
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode DAC_SetRange1(TBoardIndex iBoard, TChannelIndex ch = ciAll, Range range = Range.NoChange);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode DAC_SetRangeAll(TBoardIndex iBoard, Int32 rangeCt, IntPtr pRangeCodes);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode DAC_OutputRaw(TBoardIndex iBoard, TChannelIndex ch = ciAll, UInt64 raw = 0xFFFFFFFFFFFFFFFF);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode DAC_OutputV(TBoardIndex iBoard, TChannelIndex ch = ciAll, Double V = 0.0);
        #endregion

        #region ADC functions
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_SetRange1(TBoardIndex iBoard, Sequencer ADC, TChannelIndex channel = ciAll, Range range = Range.Bipolar10);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_SetRangeAll(TBoardIndex iBoard, Sequencer ADC, UInt32 rangeCt, IntPtr ranges);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_SetRate(TBoardIndex iBoard, Sequencer ADC, double Hz);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_GetRate(TBoardIndex iBoard, Sequencer ADC, ref double Hz);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_SetHighestChannel(TBoardIndex iBoard, Sequencer ADC, TChannelIndex lastCh);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_GetImmediateV(TBoardIndex iBoard, Sequencer ADC, out double V, TChannelIndex ch, Range range = Range.NoChange);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_ConfigureAcquire(TBoardIndex iBoard, Sequencer ADC, ChunkingType chunkingType, UInt32 chunk); 
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_AcquireBlock(TBoardIndex iBoard, Sequencer ADC, int sampleCt);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_AcquireStream(TBoardIndex iBoard, Sequencer ADC);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_SetCallback(TBoardIndex iBoard, Sequencer ADC, CallbackType callbackType, IntPtr callback);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode ADC_GetDataV(TBoardIndex iBoard, Sequencer ADC, UInt32 dataCt, out double[] data, out UInt16[] status, out UInt16[] timestamps);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode ADC_GetDataRaw(TBoardIndex iBoard, Sequencer ADC, UInt32 dataCt, out UInt16[] data, out UInt16[] status, out UInt16[] timestamps);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode ADC_PollData(TBoardIndex iBoard, Sequencer ADC);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode ADC_ClearData(TBoardIndex iBoard, Sequencer ADC);
        #endregion

        #region Low Level register read/write functions
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern byte PIO_In8(TBoardIndex iBoard, UInt32 offset);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern UInt16 PIO_In16(TBoardIndex iBoard, UInt32 offset);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern UInt32 PIO_In32(TBoardIndex iBoard, UInt32 offset);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode PIO_Out8(TBoardIndex iBoard, UInt32 offset, byte data);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode PIO_Out16(TBoardIndex iBoard, UInt32 offset, UInt16 data);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode PIO_Out32(TBoardIndex iBoard, UInt32 offset, UInt32 data);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
         internal static extern TWinErrCode PIO_INS8(TBoardIndex iBoard, UInt32 offset, UInt32 dataCt, in byte[] data);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode PIO_INS16(TBoardIndex iBoard, UInt32 offset, UInt32 dataCt, in UInt16[] data);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode PIO_INS32(TBoardIndex iBoard, UInt32 offset, UInt32 dataCt, in UInt32[] data);
        [DllImport("AIOAIO.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        internal static extern TWinErrCode PIO_WaitForIRQ(TBoardIndex iBoard, IRQIndex irq=IRQIndex.Only);
        #endregion

        #region Reserved for future implementation

        // reserved for future implementation
        //function BRD_SleepDevice			(iBoard: TBoardIndex): TWinErrCode;
        //function BRD_GetID			(iBoard: TBoardIndex; pID: TPUInt64): TWinErrCode;
        //function BRD_GetRev			(iBoard: TBoardIndex; pRev: TPUInt32): TWinErrCode;

        //function DAC_OutputVAll			(iBoard: TBoardIndex; Ct: TUInt32; V: TPDouble): TWinErrCode;
        //function DAC_OutputAll			(iBoard: TBoardIndex; Ct: TUInt32; raw: TUInt64): TWinErrCode;
        // header files overload DAC_OutputVAll with DAC_OutputAll			() as DAC_OutputAll			() -- or as DAC_Output			() for all four, if it could be done
        #endregion
    }
}
